/**
 * Licensed to jclouds, Inc. (jclouds) under one or more
 * contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  jclouds licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.jclouds.chef;

import java.io.Closeable;

import org.jclouds.View;
import org.jclouds.chef.internal.ChefContextImpl;

import com.google.inject.ImplementedBy;

/**
 * Provides an entry point to Chef features.
 * 
 * @author Ignasi Barrera
 */
@ImplementedBy(ChefContextImpl.class)
public interface ChefContext extends View, Closeable {

   /**
    * Provides access to high level Chef features.
    */
   ChefService getChefService();

   /**
    * Provides access to the underlying Chef api.
    */
   <A extends ChefApi> A getApi(Class<A> apiClass);

}
